#include "widget.h"
#include "ui_widget.h"
#include <QRect>
#include <QRgb>
#include <QDebug>
#include <QScreen>


#define FRAMECOLS 4             // nombre de colones d'images sur l'interface
#define FRAMELINES 2            // nombre de lignes d'images sur l'interface

//________________________________________________________________________________________________________
Widget::Widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Widget)
{
ui->setupUi(this);

// le QGridLayout a été posé sur l'interface; on l'associe à la fenetre principale
// ce qui fait qu'il s'ajustera automatiquement à la taille de cette dernière
this->setLayout(ui->gridLayout);

// construction de l'interface contenant plusieurs ImageFrame rangés en lignexcolonne
for (int i=0 ; i<FRAMECOLS ; i++)
    for (int j=0 ; j<FRAMELINES ; j++){
        ImageFrame *f = new ImageFrame();
        f->setFrameStyle(QFrame::Sunken | QFrame::Box);     // style voir QFrame
         ui->gridLayout->addWidget(f, j, i);                // ajout à l'interface
        frames.push_back(f);                                // ajout dans la liste
    }
// la dernière case contient des controles (bouton, slider, .etc.
ui->gridLayout->addWidget(ui->verticalLayoutWidget, FRAMELINES, 0, -1, -1, Qt::AlignCenter);

// le slider a une taille minimum de la moitié de la résolution de l'écran principal: JUST FOR FUN !
ui->horizontalSlider->setMinimumWidth(QApplication::screens()[0]->geometry().width()/2);

// instanciation de la caméra et de l'imageCapture
cam = new QCamera(this);
imageCapture = new QCameraImageCapture(cam);
// quand une image est capturée, on exécute la méthode imageCaptured
connect(imageCapture, SIGNAL(imageCaptured(int,QImage)), this, SLOT(imageCaptured(int,QImage)));
// instanciation du timer et connexion de son évènement
timer = new QTimer();
connect(timer, SIGNAL(timeout()), this, SLOT(onTimer()));

// une seule image à la fois
cam->setCaptureMode(QCamera::CaptureStillImage);
cam->start();

// titre de la fenêtre
setWindowTitle("Capture d'image - version 1.00");
}

//________________________________________________________________________________________________________
// destructeur de la fenêtre
Widget::~Widget()
{
delete ui;
cam->stop(); delete cam;
delete imageCapture;
timer->stop(); delete timer;
}

//________________________________________________________________________________________________________
//                  slot du timer
void Widget::onTimer(){
    //on prend une image
    cam->searchAndLock();
    imageCapture->capture();
    cam->unlock();
}

//________________________________________________________________________________________________________
void Widget::on_pushButton_clicked()
{
    // lancement de l'acquisition
    timer->start(100);
}

//___________________________________________
//                                  slot: une image a été capturée
void Widget::imageCaptured(int n, QImage image){
    if (image.isNull()) return;

    // limage originale sur la première case
    frames[0]->set_image(image);

    // extraction des 3 composantes RGB
     QImage redcomp(image.size(), image.format());
     QImage greencomp(image.size(), image.format());
     QImage bluecomp(image.size(), image.format());
    for (int x=0 ; x<image.width() ; x++)
        for (int y=0 ; y<image.height() ; y++){
            QRgb value = image.pixel(x,y);
            int red = qRed(value);
            int blue = qBlue(value);
            int green = qGreen(value);
            redcomp.setPixel(x,y, qRgb(red,0,0));
            greencomp.setPixel(x,y, qRgb(0,green,0));
            bluecomp.setPixel(x,y, qRgb(0,0, blue));
            }
    // les trois composantes RGB sur les cases suivantes
    frames[1]->set_image(redcomp);
    frames[2]->set_image(greencomp);
    frames[3]->set_image(bluecomp);

    // l'image en N&B sur la case suivante
    QImage bw = image.convertToFormat(QImage::Format_Grayscale8);      // la même en N&B
    frames[4]->set_image(bw);

    // l'image inversée (négatif) sur la case suivante
    QImage inv = bw;
    inv.invertPixels();
    frames[5]->set_image(inv);

    // quelques seuillages
    QImage seuilBas = bw;
    QImage seuilHaut = bw;
    for (int x=0 ; x<image.width() ; x++)
        for (int y=0 ; y<image.height() ; y++){
            QRgb value = bw.pixel(x,y);
            int grayValue = qGray(value);
            // tous les pixel > à la valeur du slider sont mis à zéro dans seuilBas
            // tout les pixels <= à la valeur du slider sont mis à zéro dans seuilHaut
            if (grayValue > ui->horizontalSlider->value())seuilBas.setPixel(x, y, 0);
            else  seuilHaut.setPixel(x, y, 0);
            int red = qRed(value);
            int blue = qBlue(value);
            int green = qGreen(value);
            redcomp.setPixel(x,y, qRgb(red,0,0));
            greencomp.setPixel(x,y, qRgb(0,green,0));
            bluecomp.setPixel(x,y, qRgb(0,0, blue));
            }
    frames[6]->set_image(seuilBas);
    frames[7]->set_image(seuilHaut);

    // dessin de l'interface, donc des images
    repaint();
}

// on a agit sur le slider
void Widget::on_horizontalSlider_sliderMoved(int position)
{

}
